﻿/**
 * Copyright 2009 Canonical Ltd.
 * 
 * This file is part of UbuntuOne on Windows.
 * 
 * UbuntuOne on Windows is free software: you can redistribute it and/or modify		
 * it under the terms of the GNU Lesser General Public License version 		
 * as published by the Free Software Foundation.		
 *		
 * Ubuntu One on Windows is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	
 * GNU Lesser General Public License for more details.	
 *
 * You should have received a copy of the GNU Lesser General Public License	
 * along with UbuntuOne for Windows.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Authors: Manuel de la Pena <manuel.delapena@canonical.com>
 */
using System;
using System.Collections.Generic;
using System.ServiceModel;
using Canonical.UbuntuOne.Common;
using log4net;

namespace Canonical.UbuntuOne.ProcessDispatcher
{
    /// <summary>
    /// Implementation of the sync daemon interface that allows to manage the service from the windows service.
    /// </summary>
    class SyncDaemonService : ISyncDaemon
    {
        #region Variables

        // contains a map between user id and pid so that we know if the process is running already.
        private readonly IDictionary<string, IProcessManager> _processesMap;
        private readonly object _loggerLock = new object();
        private ILog _logger;

        #endregion

        #region Properties

        /// <summary>
        /// Allows to get and set the logger that will be used.
        /// </summary>
        internal ILog Logger
        {
            get
            {
                if (_logger == null)
                {
                    lock (_loggerLock)
                    {
                        _logger = LogManager.GetLogger(typeof(SyncDaemonService));
                    }
                }
                return _logger;
            }
            set
            {
                _logger = value;
            }
        }

        /// <summary>
        /// Allows to get and set the manager that will manage the sync daemon process.
        /// </summary>
        internal IProcessManagerFactory ProcessManagerFactory { get; set; }

        #endregion

        public SyncDaemonService()
        {
            _processesMap = new Dictionary<string, IProcessManager>();
        }

        /// <summary>
        /// Allows to start the daemon by a client.
        /// </summary>
        /// <exception cref="SyncDaemonException">Exception thrown when the process could not be started.</exception>
        public void Start()
        {
            if (ServiceSecurityContext.Current.PrimaryIdentity.IsAuthenticated)
            {
                var user = ServiceSecurityContext.Current.PrimaryIdentity.Name;
                if (!_processesMap.ContainsKey(user))
                {
                    Logger.DebugFormat("Starting daemon for {0}.", user);
                    var manager = ProcessManagerFactory.GetManager();
                    try
                    {
                        // start the process and relate it to the user.
                        manager.Start();
                        _processesMap.Add(user, manager);
                    }
                    catch (ProcessManagementException ex)
                    {
                        var message = String.Format("Daemon for {0} could not be started: {1}", user, ex.Message);
                        Logger.ErrorFormat(message);
                        throw new SyncDaemonException(message, ex);
                    }
                }
                else
                {
                    Logger.WarnFormat("User {0} sync daemon is already running.", user);
                }
            }
        }

        /// <summary>
        /// Allows to try and start a daemon from the client. If the daemon cannot be started a false will be returned.
        /// </summary>
        /// <returns>A boolean flag telling if the daemon could be started.</returns>
        public bool TryStart()
        {
            // depend on the start method
            try
            {
                Start();
                return true;
            }
            catch(SyncDaemonException)
            {
                return false;
            }
        }

        /// <summary>
        /// Allows to stop the daemon by a client.
        /// </summary>
        /// <exception cref="SyncDaemonException">Exception thrown when the process could not be stopped.</exception>
        public void Stop()
        {
            if (ServiceSecurityContext.Current.PrimaryIdentity.IsAuthenticated)
            {
                var user = ServiceSecurityContext.Current.PrimaryIdentity.Name;
                if(_processesMap.ContainsKey(user))
                {
                    // get the manager, stop the process and remove the info
                    var manager = _processesMap[user];
                    try
                    {
                        manager.Stop();
                        _processesMap.Remove(user);
                    }
                    catch(ProcessManagementException ex)
                    {
                        var message = String.Format("Daemon for {0} could not be stoped: {1}", user, ex.Message);
                        throw new SyncDaemonException(message, ex);
                    }
                }
                else
                {
                    Logger.WarnFormat("User {0} sync daemon is not running.", user);
                }
            }
        }

        /// <summary>
        /// Allows to try and stop the daemon from the client. If the daemon cannot be stopped a false will be returned.
        /// </summary>
        /// <returns>A boolean flag telling if the daemong could bs stopped.</returns>
        public bool TryStop()
        {
            try
            {
                Stop();
                return true;
            }
            catch(SyncDaemonException)
            {
                return false;
            }
        }
    }
}
